#include "Button.h"
#include "Timer.h"
#include <EEPROM.h>

#define DEBUG

#define LED_PIN 3
#define EEPROM_INITIALIZED_FLAG 42 //Just a random value (maybe the answer to all questions)
// STATE Data
enum state{
  IDLE,
  SWIPING_LEFT,
  SWIPING_RIGHT,
  INCREASING_BRIGHTNESS,
  DECREASING_BRIGHTNESS
};

state currentState = IDLE;
uint8_t currentLevel = 0;

int rightSwipeCounter = 0;
int leftSwipeCounter = 0;

// BRIGHTNESS Data
#define NUMBER_OF_LEVELS 4
uint8_t currentBrightness = 0;
uint8_t brightnessLevels[NUMBER_OF_LEVELS]={50,100,180,255};

// OBJECTS
Timer timeoutTimer(1000);
MultiTapButton minusButton(4);
MultiTapButton centerButton(5);
MultiTapButton plusButton(6);

// FUNCTION PROTOTYPES
bool eepromInitialized();
void initializeEeprom();
void loadSettings();
void saveBrightness();

void updateButtons();
void processUserInput();

void updateLeds();
void increaseLevel();
void decreaseLevel();

void setup() {
#ifdef DEBUG
  Serial.begin(9600);
  Serial.println("Hello World");
#endif
pinMode(LED_PIN, OUTPUT);
if(eepromInitialized()){
  loadSettings();
}
else{
  initializeEeprom();
}

currentBrightness = brightnessLevels[currentLevel];
updateLeds();
}

void loop() {
 updateButtons();
 
 if(timeoutTimer.isFinished()){
  #ifdef DEBUG
    Serial.println("Timeout Reached");
  #endif
  timeoutTimer.stop();
  leftSwipeCounter = 0;
  rightSwipeCounter = 0;
  currentState = IDLE;
 }
 
 processUserInput();
}

bool eepromInitialized(){
  return (EEPROM.read(0) == EEPROM_INITIALIZED_FLAG);
}

void initializeEeprom(){
  for (int i = 0 ; i < EEPROM.length() ; i++) {
    EEPROM.write(i, 0);
  }
  
  EEPROM.write(0,EEPROM_INITIALIZED_FLAG);
  
  for(int i = 0; i < NUMBER_OF_LEVELS; i++){
    EEPROM.write(i+2, brightnessLevels[i]);
  }
}

void loadSettings(){
  currentLevel = EEPROM.read(1);
  for(int i = 0; i < NUMBER_OF_LEVELS; i++){
   brightnessLevels[i] = EEPROM.read(i+2);
  }
}

void saveBrightness(){
  brightnessLevels[currentLevel] = currentBrightness;
  EEPROM.update(currentLevel+2,brightnessLevels[currentLevel]);
}

void updateButtons(){
  minusButton.update();
  centerButton.update();
  plusButton.update();
}

void processUserInput(){
  switch(currentState){
    case IDLE: 
      if(minusButton.wasTapped()){
        rightSwipeCounter = 1;
        currentState = SWIPING_RIGHT;
        timeoutTimer.start();
        break;
      }
      if(plusButton.wasTapped()){
        leftSwipeCounter = 1;
        currentState = SWIPING_LEFT;
        timeoutTimer.start();
        break;
      }
      if(minusButton.wasLongPressed()){
        currentState = DECREASING_BRIGHTNESS;
        break;
      }
      if(plusButton.wasLongPressed()){
        currentState = INCREASING_BRIGHTNESS;
        break;
      }
      if(centerButton.wasLongPressed()){
        saveBrightness();
        break;
      }
        break;
    case SWIPING_RIGHT:
      if(centerButton.wasTapped() && (rightSwipeCounter == 1) ){
        rightSwipeCounter = 2;
        break;
      }
      if(plusButton.wasTapped() && (rightSwipeCounter == 2) ){
        rightSwipeCounter = 0;
        increaseLevel();
        timeoutTimer.stop();
        currentState = IDLE;
        break;
      }
      break;
      
    case SWIPING_LEFT:
      if(centerButton.wasTapped() && (leftSwipeCounter == 1) ){
        leftSwipeCounter = 2;
        break;
      }
      if(minusButton.wasTapped() && (leftSwipeCounter == 2) ){
        leftSwipeCounter = 0;
        decreaseLevel();
        timeoutTimer.stop();
        currentState = IDLE;
        break;
      }
      break;

    case DECREASING_BRIGHTNESS:
      if( minusButton.isHeldDown() ){
        if(currentBrightness > 0){
        currentBrightness -= 1;
        updateLeds();
        }
      }
      else{
        currentState = IDLE;
      }
      break;
      
    case INCREASING_BRIGHTNESS:
      if( plusButton.isHeldDown()){
        if(currentBrightness < 255){
        currentBrightness += 1;
        updateLeds();
        }
      }
      else{
        currentState = IDLE;
      }
      break;

  }
      
}

void updateLeds(){
  analogWrite(LED_PIN, 255 - currentBrightness); //Output is negated
  Serial.println("Updated LEDs to " + String(currentBrightness) );
}

void increaseLevel(){
  currentLevel += 1;
  currentLevel %= NUMBER_OF_LEVELS;
  currentBrightness = brightnessLevels[currentLevel];
  updateLeds();
  EEPROM.write(1, currentLevel);
  Serial.println("Increased Level to " + String(currentLevel) );
}

void decreaseLevel(){
  if(currentLevel > 0){
    currentLevel -= 1;
  }
  else{
    currentLevel = NUMBER_OF_LEVELS - 1;
  }
  currentBrightness = brightnessLevels[currentLevel];
  updateLeds();
  EEPROM.write(1, currentLevel);
  Serial.println("Decreased Level to " + String(currentLevel) );
}
